﻿#region License Information
/* HeuristicLab
 * Copyright (C) Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Drawing;
using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;

namespace HeuristicLab.Data {
  [Item("EnumValue", "An abstract base class for representing values of enum types.")]
  [StorableType("1D2E4514-1B3B-4498-9A1F-3D90C69635CA")]
  public sealed class EnumValue<T> : ValueTypeValue<T>, IComparable<EnumValue<T>> where T : struct, IComparable {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Enum; }
    }

    static EnumValue() {
      if (!typeof(T).IsEnum)
        throw new InvalidOperationException("Generic type " + typeof(T).Name + " is not an enum.");
    }

    public EnumValue() : base(default(T)) { }
    public EnumValue(T value, bool @readonly = false) : base(value, @readonly) { }

    [StorableConstructor]
    private EnumValue(StorableConstructorFlag _) : base(_) { }
    private EnumValue(EnumValue<T> original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new EnumValue<T>(this, cloner);
    }

    public int CompareTo(EnumValue<T> other) {
      return Value.CompareTo(other.Value);
    }

    public new EnumValue<T> AsReadOnly() {
      return (EnumValue<T>)base.AsReadOnly();
    }
  }
}
